﻿using Infragistics.Controls.Editors;
using Infragistics.Documents.RichText;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Documents;
using System.Windows.Media;

namespace xamRichTextEditorOfficeFormatToolbar
{
    /// <summary>
    /// Interaction logic for RichTextFormatBar.xaml
    /// </summary>
    public partial class RichTextFormatBar : UserControl, IRichTextFormatBar
    {
        bool _updatingVisualState;

        #region Properties

        public static double[] FontSizes
        {
            get
            {
                return new double[] { 
		            3.0, 4.0, 5.0, 6.0, 6.5, 7.0, 7.5, 8.0, 8.5, 9.0, 9.5, 
		            10.0, 10.5, 11.0, 11.5, 12.0, 12.5, 13.0, 13.5, 14.0, 15.0,
		            16.0, 17.0, 18.0, 19.0, 20.0, 22.0, 24.0, 26.0, 28.0, 30.0,
		            32.0, 34.0, 36.0, 38.0, 40.0, 44.0, 48.0, 52.0, 56.0, 60.0, 64.0, 68.0, 72.0, 76.0,
		            80.0, 88.0, 96.0, 104.0, 112.0, 120.0, 128.0, 136.0, 144.0
		            };
            }
        }

        public static readonly DependencyProperty TargetProperty = DependencyProperty.Register("Target", typeof(XamRichTextEditor), typeof(RichTextFormatBar));
        public XamRichTextEditor Target
        {
            get { return (XamRichTextEditor)GetValue(TargetProperty); }
            set { SetValue(TargetProperty, value); }
        }

        #endregion //Properties

        #region Ctor

        public RichTextFormatBar()
        {
            InitializeComponent();

            _cmbFontFamilies.ItemsSource = Fonts.SystemFontFamilies.ToList().Select(x => x.Source);
            _cmbFontSizes.ItemsSource = FontSizes;
        }

        #endregion //Ctor

        #region Events

        private void DragWidget_DragDelta(object sender, DragDeltaEventArgs e)
        {
            ProcessMove(e);
        }

        private void FontFamily_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0 || _updatingVisualState)
                return;

            var font = e.AddedItems[0];
            Target.Selection.ApplyFont(new Infragistics.Documents.RichText.RichTextFont(font.ToString()));
        }

        private void FontSize_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0 || _updatingVisualState)
                return;

            var size = (double)e.AddedItems[0];
            Target.Selection.ApplyFontSize(size);
        }

        #endregion //Events

        #region Methods

        /// <summary>
        /// Processes the dragging of the RichTextFormatBar
        /// </summary>
        private void ProcessMove(DragDeltaEventArgs e)
        {
            AdornerLayer layer = AdornerLayer.GetAdornerLayer(Target);
            UIElementAdorner<Control> adorner = layer.GetAdorners(Target)[0] as UIElementAdorner<Control>;
            adorner.SetOffsets(adorner.OffsetLeft + e.HorizontalChange, adorner.OffsetTop + e.VerticalChange);
        }

        /// <summary>
        /// Updates the visual state of the buttons and other tools of the RichTextFormatBar
        /// </summary>
        public void UpdateVisualState()
        {
            _updatingVisualState = true;

            DocumentSpan documentSpan = Target.Selection == null ? new DocumentSpan(0, 0) : Target.Selection.DocumentSpan;
            var settings = Target.Document.GetCommonCharacterSettings(documentSpan);
            UpdateSelectedFontFamily(settings);
            UpdateSelectedFontSize(settings);

            UpdateBoldState(settings);
            UpdateItalicState(settings);
            UpdateUnderlineState(settings);

            UpdateParagraphAlignmentState(documentSpan);

            _updatingVisualState = false;
        }

        /// <summary>
        /// Updates the selected item in the font family list.
        /// </summary>
        void UpdateSelectedFontFamily(CharacterSettings settings)
        {
            if (settings.FontSettings != null)
            {
                _cmbFontFamilies.SelectedValue = (settings.FontSettings.Ascii.HasValue && !string.IsNullOrWhiteSpace(settings.FontSettings.Ascii.Value.Name)) ? settings.FontSettings.Ascii.Value.Name : "Arial";
            }
        }

        /// <summary>
        /// Updates the selected item in the font list.
        /// </summary>
        void UpdateSelectedFontSize(CharacterSettings settings)
        {
            _cmbFontSizes.SelectedValue = settings.FontSize.HasValue ? settings.FontSize.Value.Points : 12.0;
        }

        /// <summary>
        /// Updates the state of the bold button
        /// </summary>
        void UpdateBoldState(CharacterSettings settings)
        {
            UpdateToggleButtonCheckedState(_btnBold, settings.Bold);
        }

        /// <summary>
        /// Updates the state of the italic button
        /// </summary>
        void UpdateItalicState(CharacterSettings settings)
        {
            UpdateToggleButtonCheckedState(_btnItalic, settings.Italics);
        }

        /// <summary>
        /// Updates the state of the underline button
        /// </summary>
        void UpdateUnderlineState(CharacterSettings settings)
        {
            if (settings.UnderlineType.HasValue)
                UpdateToggleButtonCheckedState(_btnUnderline, settings.UnderlineType.Value != UnderlineType.None);
        }

        /// <summary>
        /// Updates the state of the paragraph alignment buttons
        /// </summary>
        void UpdateParagraphAlignmentState(DocumentSpan documentSpan)
        {
            ParagraphSettings paragraphSettings = Target.Document.GetCommonParagraphSettings(documentSpan);
            if (paragraphSettings.ParagraphAlignment.HasValue)
            {
                var value = paragraphSettings.ParagraphAlignment.Value;
                switch (value)
                {
                    case ParagraphAlignment.Start:
                        {
                            UpdateToggleButtonCheckedState(_btnAlignLeft, true);
                            break;
                        }
                    case ParagraphAlignment.Center:
                        {
                            UpdateToggleButtonCheckedState(_btnAlignCenter, true);
                            break;
                        }
                    case ParagraphAlignment.End:
                        {
                            UpdateToggleButtonCheckedState(_btnAlignRight, true);
                            break;
                        }
                }
            }
        }

        /// <summary>
        /// Updates the visual state of a toggle button.
        /// </summary>
        /// <param name="button">The button.</param>
        /// <param name="formattingProperty">The formatting property.</param>
        /// <param name="expectedValue">The expected value.</param>
        void UpdateToggleButtonCheckedState(ToggleButton button, bool? value)
        {
            button.IsChecked = value.HasValue ? value.Value : false;
        }

        #endregion //Methods
    }
}
